--[[
Name:		  GJBPlayer-1.0
Author(s): 	  Azmaedus
Revision: 	  $Rev: 1 $
Created:	  01/23/2015
Description:  Library creates a music player for GJB.
Dependencies: LibStub
License:	  GPLv2

	Change Log
	--------------------
	01/23/2015 - 1.0a 	Creation of addon
]]

local GJBPLAYER = "GJBPlayer-1.0"
local GJBPLAYER_MINOR = 1 + tonumber(("$Revision: 1 $"):match("(%d+)"))

local GJBP = LibStub:NewLibrary(GJBPLAYER, GJBPLAYER_MINOR)
if not GJBP then return end

GJBPdefaults = {
	-- Background
	BG = {
		-- Backdrop info
		BD = {
			bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
			edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
			tile = true, tileSize = 4, edgeSize = 4,
			insets = { left = 1, right = 1, top = 1, bottom = 1 }
		},
		align = "CENTER", xPos = 0, yPos = 0, Scale = 1, w = 200,	h = 150, r = 0,	g = 0, b = 0, a = 1, br = 1, bg = 1, bb = 1, ba = 1,
	},
	CtrlButtons = {	align = "BOTTOMCENTER",	xPos = 0, yPos = 0, Scale = 1, space = 1, w = 40, h = 20, r = 0, g = 0, b = 0, a = 1, br = 1, bg = 1, bb = 1, ba = 1 },
}

if not GJBP.frame then
	GJBP.frame = CreateFrame("Frame")
	GJBP.frame:SetWidth(GJBPdefaults.BG.w)
	GJBP.frame:SetHeight(GJBPdefaults.BG.h)
	GJBP.frame:SetBackdrop(GJBPdefaults.BG.BD)
	GJBP.frame:SetBackdropColor(GJBPdefaults.BG.r, GJBPdefaults.BG.g, GJBPdefaults.BG.b, GJBPdefaults.BG.a)
	GJBP.frame:SetBackdropBorderColor(GJBPdefaults.BG.br, GJBPdefaults.BG.bg, GJBPdefaults.BG.bb, GJBPdefaults.BG.ba)
	GJBP.frame.pos = {}
	GJBP.frame:Show()
end

local L_GJBP_TITLE = "GarrisonJukeBox Music Player"
local L_GJBP_FRAME_ERROR = "Error creating frame"

local l = GetLocale()
if l == "frFR" then
	L_GJBP_TITLE = "Lecteur GarrisonJukeBox"
	L_GJBP_FRAME_ERROR = "Erreur lors de la creation de la frame"
elseif l == "koKR" then
	-- Missing
elseif l == "deDE" then
	-- Missing
elseif l == "zhCN" then
	-- Missing
elseif l == "zhTW" then
	-- Missing
elseif l == "ruRU" then
	-- Missing
elseif l == "esES" or l == "esMX" then
	-- Missing
elseif l == "ptBR" then
	-- Missing
elseif l == "itIT" then
	-- Missing
end

-- GJBP publics
GJBP.tframes = {}

-- Function to create a sub-frame
function GJBP:genFrame(f, parent)
	if not f then return end -- fail
	local fo = {}
	if parent == UIParent then
		fo = CreateFrame(f.ftype, f.name, UIParent)
	else
		fo = CreateFrame(f.ftype, f.name, parent, f.template)
	end
	fo:SetWidth(f.dim.w)
	fo:SetHeight(f.dim.h)
	if f.bd then
		fo:SetBackdrop(f.bd)
		fo:SetBackdropColor(f.rgba.r, f.rgba.g, f.rgba.b, f.rgba.a)
		fo:SetBackdropBorderColor(f.brgba.r, f.brgba.g, f.brgba.b, f.brgba.a)
	end
	if f.show then
		fo:Show()
	end
	return fo
end

--[[
Creates all sub frames by passing a table of frames to create
Structure of the tframes table:
	t = {
		{
			ftype = "type", --Frame, Button, ...
			rgba = {r = 1, g = 1, b = 1, a = 1}, -- red, green, blue, alpha 0.0 to 1.0
			dim = {w = 1, h = 1, s = 1}, -- width, height, scale
			pos = {x = 0, y = 0, a = "ANCHOR", r = "PARENT", ap = "PARENT-ANCHOR"}, -- x, y, anchor, relative to, r anchor
			bd = {}, -- BackDrop
			brgba = {r = 1, g = 1, b = 1, a = 1}, -- BackDrop : red, green, blue, alpha 0.0 to 1.0
			show = true,
			movable = {
				enabled = false, 
				drag = {
					"LeftButton",
					...
				}
			},
			clicks = {"LeftButton","RightButton","Button4"},
			scripts = {
				{event = "OnSomething", func = function(self,...)end},
				...
			},
		},
		...
	}
]]
GJBP.prev = UIParent

function GJBP:makeFrames(t)
GJBP.tframes = {}
	for i=1,#t do
		-- check if frame exists
		if t[i] then
			GJBP.tframes[i] = GJBP:genFrame(t[i], GJBP.prev)
			GJBP.prev = GJBP.tframes[i]
			GJBP.tframes[i]:SetPoint(t[i].pos.a, t[i].pos.r, t[i].pos.rp, t[i].pos.x, t[i].pos.y)
			--print(t[i].pos.a, t[i].pos.r, t[i].pos.ap, t[i].pos.x, t[i].pos.y)
			
			if t[i].text then
				GJBP.tframes[i].text = GJBP.tframes[i]:CreateFontString(nil, "ARTWORK", "NumberFont_Shadow_Small")
				GJBP.tframes[i].text:SetAllPoints(true)
				GJBP.tframes[i].text:SetJustifyH("CENTER")
				GJBP.tframes[i].text:SetJustifyV("CENTER")
				GJBP.tframes[i].text:SetTextColor(1,1,1,1)
				GJBP.tframes[i].text:SetText(t[i].text)
			end
			
			-- if frame is movable
			if t[i].movable and t[i].movable.enabled then
				GJBP.tframes[i]:SetMovable(true)
				GJBP.tframes[i]:EnableMouse(true)
				GJBP.tframes[i]:RegisterForDrag(unpack(t[i].movable.drag))
				GJBP.tframes[i]:SetScale(t[i].dim.s)
			end

			-- register click events
			if t[i].clicks then
				GJBP.tframes[i]:SetPoint("CENTER")
				GJBP.tframes[i]:RegisterForClicks(unpack(t[i].clicks))
				--GJBP.tframes[i]:SetNormalTexture("\\media\\button.tga")
			end
			
			-- create scripts
			if t[i].scripts then
				for y=1,#t[i].scripts do
					GJBP.tframes[i]:SetScript(t[i].scripts[y].event, t[i].scripts[y].func)
				end
			end
		else
			assert(false, L_GJBP_FRAME_ERROR .. " #" .. i) -- Error
		end
	end
end

function GJBP:setVisible( visible )
	if visible then
		for i = 1, #GJBP.tframes do
			GJBP.tframes[i]:Show()
		end
	else
		for i = 1, #GJBP.tframes do
			GJBP.tframes[i]:Hide()
		end
	end
end

function GJBP:updateFrames()
	for i = 1, #GJBP.tframes do
		--GJBP.buttons["gjbpframe"..i]:SetBackdropColor( red, green, blue, GJBPoptions.BackDropAlpha )
	end
end

function GJBP:loadOptions()
	return self.GJBPoptions
end

function GJBP:saveOptions(t)
	self.GJBPoptions = t
end

function GJBP:setPosition(a, x, y)
	GJBP.tframes[1]:SetPoint(a, p, nil, x, y)
end

local function OnUpdate()
	GJBP:updateFrames()
end

GJBP.frame:SetScript("OnUpdate", OnUpdate);